unit crt;

{

http://www.freepascal.org/docs-html/rtl/crt/index-5.html

Consol
ClrEol
ClrScr
CursorOff
CursorOn
Delay
DelLine
GotoXY
InsLine
Keypressed
NoSound
ReadKey
Sound
TextBackground
TextColor
WhereX
WhereY

}

interface

var
	TextAttr: byte = 0;
	Consol: byte absolute $d01f;
	WhereY: byte absolute $54;		// Return Y (vertical) cursor position
	WhereX: word absolute $55;		// Return X (horizontal) cursor position

const
	CN_START_SELECT_OPTION	= 0;		// Consol value
	CN_SELECT_OPTION	= 1;
	CN_START_OPTION		= 2;
	CN_OPTION		= 3;
	CN_START_SELECT		= 4;
	CN_SELECT		= 5;
	CN_START		= 6;
	CN_NONE			= 7;
 
implementation


procedure CursorOff;
//----------------------------------------------------------------------------------------------
// Hide cursor
//----------------------------------------------------------------------------------------------
begin

asm
{	mva #1 crsinh
};
 
 write( #31#30 );	// CH_CURS_RIGHT, CH_CURS_LEFT

end;


procedure CursorOn;
//----------------------------------------------------------------------------------------------
// Display cursor
//----------------------------------------------------------------------------------------------
begin

asm
{	mva #0 crsinh
};
 
 write( #31#30 );

end;


procedure ClrScr;
//----------------------------------------------------------------------------------------------
// Clear screen
//----------------------------------------------------------------------------------------------
begin

 write( #$7d );		// CH_CLR

end;


procedure DelLine;
//----------------------------------------------------------------------------------------------
// Delete line at cursor position
//----------------------------------------------------------------------------------------------
begin

 write( #$9c );		// CH_DELLINE

end;


procedure InsLine;
//----------------------------------------------------------------------------------------------
// Insert an empty line at cursor position
//----------------------------------------------------------------------------------------------
begin

 write( #$9d );		// CH_INSLINE

end;


function ReadKey: char; assembler;
asm
{	txa:pha

	@GetKey

	sta Result

	pla:tax
};
end;


procedure TextBackground(a: byte); assembler;
asm
{	mwa a colpf2s
};
end;


procedure TextColor(a: byte); assembler;
asm
{	mva a colpf1s
};
end;


procedure Delay(count: word); assembler;
asm
{	txa:pha

	ldx #0
	ldy #0

loop	cpy count
	bne @+
	cpx count+1
	beq stop

@	:16 sta wsync

	iny
	sne
	inx

	bne loop

stop	pla:tax
};
end;


function Keypressed: Boolean; assembler;
asm
{	ldy #$00	; false
	lda kbcodes
	cmp #$ff
	beq skp
	iny		; true

;	sty kbcodes

skp	sty Result
};
end;


procedure GotoXY(x,y: byte); assembler;
//----------------------------------------------------------------------------------------------
// Set cursor position on screen
//
// GotoXY positions the cursor at (X,Y), X in horizontal, Y in vertical direction relative to
// the origin of the current window. The origin is located at (1,1), the upper-left corner
// of the window.
//----------------------------------------------------------------------------------------------
asm
{	ldy x
	beq @+
	dey
	sty colcrs

@	ldy y
	beq @+
	dey
	sty rowcrs
@
};
end;


procedure ClrEol;
//----------------------------------------------------------------------------------------------
// ClrEol clears the current line, starting from the cursor position, to the end of the window.
// The cursor doesn't move
//----------------------------------------------------------------------------------------------
begin
	FillChar( pointer(DPeek(88)+WhereX+WhereY*40), 40-WhereX, 0);
end;


procedure NoSound; assembler;
asm
{	jsr @rstsnd

	ldy #8
lp	sta $d200,y
	sta $d210,y
	dey
	bpl lp
};
end;


procedure Sound(Chan,Freq,Dist,Vol: byte); assembler;
asm
{	jsr @rstsnd

	lda Chan
	and #3
	asl @
	tay

	lda Vol
	and #$0F
	sta _t
 
	lda Dist
	:4 asl @
	ora #0
_t	equ *-1

	sta $d201,y

	lda Freq
	sta $d200,y
};
end;


end.

