unit graph;

{

http://www.freepascal.org/docs-html/rtl/graph/index-5.html

Circle
Ellipse
FillEllipse
FillRect
FloodFill
GetPixel
InitGraph
Line
LineTo
MoveTo
PutPixel
Rectangle
SetBkColor
SetColor
SetColorMapDimensions
SetColorMapEnrty
SetCurrentPaletteEntry
SetPaletteEntry
SetRGBPalette

}


interface

var
	GraphResult: byte;

const
	D2bit = 12;
	D4bit = 13;
	D6bit = 14;
	D8bit = 15;

	m640x480 = 8 + 16;
	
	grOK = 0;

implementation



procedure InitGraph(mode: byte); overload;
//----------------------------------------------------------------------------------------------
// init graphics mode
//----------------------------------------------------------------------------------------------
var window: byte;
begin

	GraphResult := 0;

	window := mode and $10;

	mode := mode and $0f;

	ScreenHeight := 192;

	case mode of
	0,3:	begin ScreenWidth := 40; ScreenHeight := 24 end;
	7:	begin ScreenWidth := 160; ScreenHeight := 96 end;
	8:	ScreenWidth := 320;
	9..11:	ScreenWidth := 80;
	15:	ScreenWidth := 160

	else
		GraphResult := 128;
	end;

asm
{	txa:pha

	mva #$2c @putchar.vbxe

	ldy mode

	ldx #$60	; 6*16
	lda window	; %00010000 with text window
	eor #$10
	ora #2		; read

	.nowarn @graphics

	pla:tax
};
end;


procedure SetBkColor(color: byte); assembler;
//----------------------------------------------------------------------------------------------
// Sets the background color to Color
//----------------------------------------------------------------------------------------------
asm
{	mva color colbaks
};
end;


procedure SetColor(color: byte); assembler;
//----------------------------------------------------------------------------------------------
// Sets the foreground color to Color
//----------------------------------------------------------------------------------------------
asm
{	mva color @COMMAND.colscr
};
end;


procedure PutPixel(x,y: smallint); assembler; overload;
//----------------------------------------------------------------------------------------------
// Puts a point at (X,Y) using color Color
//----------------------------------------------------------------------------------------------
asm
{	txa:pha

	cpw y main.system.ScreenHeight
	bcs stop

	mwa x colcrs
	mva y rowcrs

	lda #@IDput

	jsr @COMMAND

stop	pla:tax
};
end;

procedure PutPixel(x,y: smallint; color: byte); overload;
//----------------------------------------------------------------------------------------------
// Puts a point at (X,Y) using color Color
//----------------------------------------------------------------------------------------------
begin

asm
{	mva color @COMMAND.colscr
};
	PutPixel(x,y);
end;


function GetPixel(x,y: smallint): byte; assembler;
//----------------------------------------------------------------------------------------------
// Return color of pixel
//----------------------------------------------------------------------------------------------
asm
{	txa:pha

	lda #$00

	cpw y main.system.ScreenHeight
	bcs stop

	mwa x colcrs
	mva y rowcrs

	lda #@IDget

	jsr @COMMAND

stop	sta Result

	pla:tax
};
end;


procedure MoveTo(x, y: smallint);
//----------------------------------------------------------------------------------------------
// Move cursor to absolute position
//----------------------------------------------------------------------------------------------
begin

 if x < 0 then x := 0;
 if y < 0 then y := 0;

asm
{	mwa x LineTo.x0
	mwa y LineTo.y0
};
end;


procedure LineTo(x, y: smallint);
//----------------------------------------------------------------------------------------------
// Draw a line starting from current position to a given point
//----------------------------------------------------------------------------------------------
begin

 if x < 0 then x := 0;
 if y < 0 then y := 0;

asm
{	.var x0 .word
	.var y0 .word

	txa:pha
		
	mwa x0 colcrs
	mva y0 rowcrs

	lda #@IDput

	jsr @COMMAND

	lda x
	sta colcrs
	sta x0
	lda x+1
	sta colcrs+1
	sta x0+1

	mva y rowcrs
	sta y0

	lda #@IDdraw

	jsr @COMMAND

	pla:tax
};
end;


procedure Line(x0, y0, x1, y1: smallint);
//----------------------------------------------------------------------------------------------
// Draw a line between 2 points
//----------------------------------------------------------------------------------------------
begin

asm
{	mwa x0 LineTo.x0
	mva y0 LineTo.y0
};
	LineTo(x1,y1);
end;


procedure FloodFill(x,y: smallint; color: byte); assembler;
//----------------------------------------------------------------------------------------------
// Fill an area with a given color
//----------------------------------------------------------------------------------------------
asm
{	txa:pha

	mva color fildat

	mwa x colcrs
	mva y rowcrs

	lda #@IDfill

	jsr @COMMAND

	pla:tax
};
end;


{$i vbxe\initgraph.pas}
{$i vbxe\colormap.pas}
{$i vbxe\setrgbpalette.pas}

{$i draw\circle.pas}
{$i draw\ellipse.pas}
{$i draw\rectangle.pas}
{$i draw\fillrect.pas}
{$i draw\fillellipse.pas}

end.

