unit math;

{

http://www.freepascal.org/docs-html/rtl/math/index-5.html

ArcCos
ArcSin
ArcTan2
Ceil
CycleToRad
DegNormalize
DegToGrad
DegToRad
DivMod
EnsureRange
Floor
GradToDeg
GradToRad
FMod
InRange
Max
Min
Power
RadToCycle
RadToDeg
RadToGrad
Tan

}

interface


implementation


function DegNormalize(deg : real) : real;
begin
  Result:=Deg-real(Trunc(Deg/360.0))*360.0;
  If Result<0.0 then Result:=Result+360.0;
end;


function degtorad(deg : real) : real;
  begin
     Result:=deg*(pi/180.0);
  end;

function radtodeg(rad : real) : real;
  begin
     Result:=rad*(180.0/pi);
  end;

function gradtorad(grad : real) : real;
  begin
     Result:=grad*(pi/200.0);
  end;

function radtograd(rad : real) : real;
  begin
     Result:=rad*(200.0/pi);
  end;

function degtograd(deg : real) : real;
  begin
     Result:=deg*(200.0/180.0);
  end;

function gradtodeg(grad : real) : real;
  begin
     Result:=grad*(180.0/200.0);
  end;

function cycletorad(cycle : real) : real;
  begin
     Result:=M_PI_2*cycle;
  end;

function radtocycle(rad : real) : real;
  begin
     { avoid division }
     Result:=rad*(1.0/M_PI_2);
  end;


procedure DivMod(Dividend: integer; Divisor: Word; var r, Remainder: Word); overload;
begin
  if Dividend < 0 then
    begin
      { Use DivMod with >=0 dividend }
	  Dividend:=-Dividend;
      { The documented behavior of Pascal's div/mod operators and DivMod
        on negative dividends is to return Result closer to zero and
        a negative Remainder. Which means that we can just negate both
        Result and Remainder, and all it's Ok. }
      r:=-(Dividend Div Divisor);
      Remainder:=-(Dividend+(r*Divisor));
    end
  else
    begin
      r:=Dividend Div Divisor;
      Remainder:=Dividend-(r*Divisor);
    end;
end;


procedure DivMod(Dividend: integer; Divisor: Word; var r, Remainder: smallint); overload;
begin
  if Dividend < 0 then
    begin
      { Use DivMod with >=0 dividend }
	  Dividend:=-Dividend;
      { The documented behavior of Pascal's div/mod operators and DivMod
        on negative dividends is to return Result closer to zero and
        a negative Remainder. Which means that we can just negate both
        Result and Remainder, and all it's Ok. }
      r:=-(Dividend Div Divisor);
      Remainder:=-(Dividend+(r*Divisor));
    end
  else
    begin
      r:=Dividend Div Divisor;
      Remainder:=Dividend-(r*Divisor);
    end;
end;


function InRange(const AValue, AMin, AMax: byte): Boolean; overload;
//----------------------------------------------------------------------------------------------
// Check whether value is in range
//----------------------------------------------------------------------------------------------
begin
  Result:=(AValue>=AMin) and (AValue<=AMax);
end;


function InRange(const AValue, AMin, AMax: Integer): Boolean; overload;
begin
  Result:=(AValue>=AMin) and (AValue<=AMax);
end;


function EnsureRange(const AValue, AMin, AMax: byte): Integer; overload;
//----------------------------------------------------------------------------------------------
// Change value to it falls in specified range
//----------------------------------------------------------------------------------------------
begin
  Result:=AValue;
  If Result<AMin then
    Result:=AMin
  else if Result>AMax then
    Result:=AMax;
end;


function EnsureRange(const AValue, AMin, AMax: Integer): Integer; overload;
begin
  Result:=AValue;
  If Result<AMin then
    Result:=AMin
  else if Result>AMax then
    Result:=AMax;
end;


function Min(x, y: real): real; overload;
//----------------------------------------------------------------------------------------------
// Return smallest of two values
//----------------------------------------------------------------------------------------------
begin
if x < y then Result := x else Result := y;
end;


function Min(x, y: integer): integer; overload;
begin
if x < y then Result := x else Result := y;
end;


function Max(x, y: real): real; overload;
//----------------------------------------------------------------------------------------------
// Return largest of two values
//----------------------------------------------------------------------------------------------
begin
if x > y then Result := x else Result := y;
end;


function Max(x, y: integer): integer; overload;
begin
if x > y then Result := x else Result := y;
end;


function power(base : real; const exponent : shortint) : real; overload;
//----------------------------------------------------------------------------------------------
// Return real power
// https://github.com/graemeg/freepascal/blob/master/rtl/objpas/math.pp
//----------------------------------------------------------------------------------------------
  var
     i : integer;
  begin
     if (base = 0.0) and (exponent = 0) then
       result:=1.0
     else
       begin
         i:=abs(exponent);
         Result:=1.0;
         while i>0 do
           begin
              while (i and 1)=0 do
                begin
                   i:=i shr 1;
                   base:=sqr(base);
                end;
              i:=i-1;
              Result:=Result*base;
           end;
         if exponent<0 then
           Result:=1.0/Result;
       end;
  end;
  

function power(base : integer; const exponent : shortint) : integer; overload;
//----------------------------------------------------------------------------------------------
// Return integer power
// https://github.com/graemeg/freepascal/blob/master/rtl/objpas/math.pp
//----------------------------------------------------------------------------------------------
  var
     i : integer;
  begin
     if (base = 0) and (exponent = 0) then
       result:=1
     else
       begin
         i:=abs(exponent);
         Result:=1;
         while i>0 do
           begin
              while (i and 1)=0 do
                begin
                   i:=i shr 1;
                   base:=sqr(base);
                end;
              i:=i-1;
              Result:=Result*base;
           end;
         if exponent<0 then
           Result:=0;
       end;
  end;


function arctan2(y,x : real) : real;
  begin
    if (x=0.0) then
      begin
        if y=0.0 then
          Result:=0.0
        else if y>0.0 then
          Result:=D_PI_2
        else if y<0.0 then
          Result:=-D_PI_2;
      end
    else
      Result:=ArcTan(y/x);
    if x<0.0 then
      Result:=Result+pi;
    if Result>pi then
      Result:=Result-M_PI_2;
  end;


function ArcSin(x: real): real;
const
 a0 = D_PI_2;
 a1 = -0.212;
 a2 = 0.074;
 a3 = -0.019;

begin

 if (x>=-1.0) and (x<=1.0) then 
  Result:= a0 - sqrt(1.0 - x)*(a0 + a1*x + a2*x*x + a3*x*x*x);
  
end;


function ArcCos(x: real): real;
begin

 if (x>=-1.0) and (x<=1.0) then 
  Result:= D_PI_2 - ArcSin(x);
  
end;


function Tan(x: Real): Real;
begin

 Result := sin(x) / cos(x);

end;


function Ceil(a: real): smallint;
begin

 Result := trunc(a - 32768.) + 32768;

end;


function Floor(a: real): smallint;
begin

 Result := trunc(a + 32768.) - 32768;

end;


function FMod(a, b: real): real;
begin

 Result := (a - b * Real(floor(a / b)));

end;

end.

