unit misc;

{

DetectCPU
DetectCPUSpeed
DetectEvie
DetectMapRam
DetectMem
DetectStereo
DetectVBXE

}

interface

var
	banks: array [0..63] of byte;


implementation


function DetectVBXE(var p: word): Boolean; assembler; register;
asm
{	txa:pha

	jsr @vbxe_detect

	ldy #0		; core
	sta (p),y

	lda fxptr+1
	sta Result

	iny		; page
	sta (p),y

	pla:tax
};
end;


function DetectEvie: Boolean; assembler;
asm
{	ldy #3
lp	lda $d2fa,y
	cmp _evie,y
	bne _no
	dey
	bpl lp

_yes	lda #true
	dta $2c

_no	lda #false
	sta Result

	jmp stop

_evie	dta c'Evie'

stop
};
end;


function DetectStereo: Boolean; assembler;
(*
 second POKEY detect routine answer in A register:
 $00 - absent
 $80 - present
 code & idea: Seban/SLIGHT
 (c) 1995,96 Slight!
*)
asm
{	txa:pha

	sei
	inc $d40e
	lda #$03
	sta $d21f
	sta $d210
	ldx #$00
	stx $d211
	inx
	stx $d21e

	ldx:rne $d40b

	stx $d219
loop	ldx $d40b
	bmi stop
	lda #$01
	bit $d20e
	bne loop

stop	lda $10
	sta $d20e
	dec $d40e
	cli
	stx Result

	pla:tax
};
end;


function DetectCPU: byte; assembler;
(*
 How to detect on which CPU the assembler code is running

 (This information is from Draco, the author of SYSINFO 2.0)

 You can test on plain 6502-Code if there is a 65c816 CPU, the 16-Bit processor avaible
 in some XLs as a turbo-board, avaible. Draco told me how to do this:

 First we make sure, whether we are running on NMOS-CPU (6502) or CMOS (65c02,65c816).
 I will just show the "official" way which doesn`t uses "illegal opcodes":

 result in regA:
 $00 - 6502
 $01 - 65c02
 $80 - 65816
*)
asm
{	txa:pha

	opt c+

;detekcja zainstalowanego procesora
	lda #$99
	clc
	sed
	adc #$01
	cld
	beq DetectCPU_CMOS

DetectCPU_02
	lda #0
	jmp stop

DetectCPU_CMOS
	lda #0
	rep #%00000010		;wyzerowanie bitu Z
	bne DetectCPU_C816

DetectCPU_C02
	lda #1
	jmp stop

DetectCPU_C816
	lda #$80

stop	sta Result

	opt c-

	pla:tax
};
end;


function DetectCPUSpeed: real;
(*
	by Draco
*)
var clkm, fr0: word;
begin

asm
{	txa:pha

	tsx
	stx	stk

	lda	vvblki
	sta	lvbl

	lda	vvblki+1
	sta	hvbl

	sei

	lda:cmp:req rtclok+2

	ldx	<stop2
	ldy	>stop2

bogo2	lda	vcount
	cmp	#112
	bne	bogo2

	stx	vvblki
	sty	vvblki+1

	lda	#$00
	sta	fr0+1
	tax
	tay

	sta	wsync

loop2	iny
	bne	loop2
	inx
	bne	loop2
	clc
	adc	#$01
	bne	loop2

stop2
	pla
	sta	clkm
	pla
	sta	clkm+1
	pla
;	sta	clkm+2
	sta	fr0

	ldx	#0
stk	equ *-1
	txs

	lda	#0
lvbl	equ *-1
	sta vvblki

	lda	#0
hvbl	equ *-1
	sta vvblki+1

	cli
	pla:tax
};
	Result := (real((fr0 shl 16) + clkm) / 487.0) * 1.7734;
end;


function DetectMEM: byte; assembler;
asm
{	txa:pha

ext_b	= $4000		;cokolwiek z zakresu $4000-$7FFF

	lda portb
	pha

	lda #$ff
	sta portb

	lda ext_b
	pha

	ldx #$0f	;zapamitanie bajtw ext (z 16 blokw po 64k)
_p0	jsr setpb
	lda ext_b
	sta bsav,x
	dex
	bpl _p0

	ldx #$0f	;wyzerowanie ich (w oddzielnej ptli, bo nie wiadomo
_p1	jsr setpb	;ktre kombinacje bitw PORTB wybieraj te same banki)
	lda #$00
	sta ext_b
	dex
	bpl _p1

	stx portb	;eliminacja pamici podstawowej
	stx ext_b
	stx $00		;niezbdne dla niektrych rozszerze do 256k

	ldy #$00	;ptla zliczajca bloki 64k
	ldx #$0f
_p2	jsr setpb
	lda ext_b	;jeli ext_b jest rne od zera, blok 64k ju zliczony
	bne _n2

	dec ext_b	;w przeciwnym wypadku zaznacz jako zliczony

	lda ext_b	;sprawdz, czy sie zaznaczyl; jesli nie -> cos nie tak ze sprzetem
	bpl _n2

	lda portb	;wpisz warto PORTB do tablicy dla banku 0
	sta adr.banks,y
	eor #%00000100	;uzupenij wartoci dla bankw 1, 2, 3
	sta adr.banks+1,y
	eor #%00001100
	sta adr.banks+2,y
	eor #%00000100
	sta adr.banks+3,y
	iny
	iny
	iny
	iny

_n2	dex
	bpl _p2

	ldx #$0f	;przywrcenie zawartoci ext
_p3	jsr setpb
	lda bsav,x
	sta ext_b
	dex
	bpl _p3

	stx portb	;X=$FF

	pla
	sta ext_b

	pla
	sta portb

	sty Result

	jmp stop

; podprogramy
setpb	txa		;zmiana kolejnoci bitw: %0000dcba -> %cba000d0
	lsr	     
	ror
	ror
	ror
	adc #$01	;ustawienie bitu nr 1 w zaleznosci od stanu C
	ora #$01	;ustawienie bitu sterujcego OS ROM na wartosc domyslna
	sta portb
	rts

bsav	.ds 16

stop	pla:tax
};
end;


function DetectMapRam: Boolean; assembler;
asm
{
; http://xxl.atari.pl/mapram/

bsav	= DX
ext_b	= $5000		;cokolwiek z zakresu $5000-$57FF

	txa:pha

	sei
	inc $d40e

	mva #FALSE Result

	lda portb
	pha

	lda #$ff
	sta portb

	lda ext_b
	pha

_p0	jsr setb
	lda ext_b
	sta bsav

	lda #$00
	sta ext_b

	lda #$ff
	sta portb	;eliminacja pamici podstawowej
	sta ext_b

_p2	jsr setb

	inc ext_b
	beq _p3

	mva #TRUE Result

_p3	lda bsav
	sta ext_b

	lda #$ff
	sta portb

	pla
	sta ext_b

	pla
	sta portb

	dec $d40e
	cli

	jmp stop	

setb	lda portb
	and #%01001110	; !!!
	ora #%00110000  ; MAPRAM ON
	sta portb	
	rts

stop	pla:tax

};
end;

end.

