unit system;

{

http://www.freepascal.org/docs-html/rtl/system/index-5.html

Abs
ArcTan
BinStr
Bounds
Cos
DPeek
DPoke
Eof
Exp
FilePos
Fillchar
HexStr
Ln
LowerCase
Move
OctStr
ParamCount
ParamStr
Pause
Peek
Poke
Point
PointsEqual 
Random
ReadConfig
ReadSector
Rect
RunError
Seek
SetLength
Sin
Space
Str
StringOfChar
Sqr
Sqrt
UpCase
Val
WriteSector

}

interface

type
	TString = string[32];

	TRect = record	left, top, right, bottom: smallint  end;
	TPoint = record x,y: SmallInt end;

var	IOResult: byte;
	ScreenWidth: word = 40;
	ScreenHeight: word = 24;

const
	M_PI_2	= 6.283285;	// pi * 2
	D_PI_2	= 1.570796;	// pi / 2
	D_PI_180= 0.017453;     // pi / 180

	GTIA = 0;
	VBXE = $80;
	WINDOW  = $10;		// text window
	NARROW  = $20;		// narrow screen

	VBXE_XDLADR = $0000;	// XDLIST
	VBXE_MAPADR = $1000;	// COLOR MAP ADDRESS
	VBXE_BCBADR = $0100;	// BLITTER LIST ADDRESS
	VBXE_OVRADR = $5000;	// OVERLAY ADDRESS
	VBXE_WINDOW = $B000;	// 4K WINDOW $B000..$BFFF

	iDLI = 0;		// set new DLI vector
	iVBL = 1;		// set new DLI vector

(* Character codes *)

	CH_DELCHR	= $FE;	// delete char under the cursor
	CH_ENTER	= $9B;
	CH_ESC		= $1B;
	CH_CURS_UP	= 28;
	CH_CURS_DOWN	= 29;
	CH_CURS_LEFT	= 30;
	CH_CURS_RIGHT	= 31;

	CH_TAB		= $7F;	// tabulator
	CH_EOL		= $9B;	// end-of-line marker
	CH_CLR		= $7D;	// clear screen
	CH_BELL		= $FD;	// bell
	CH_DEL		= $7E;	// back space (delete char to the left)
	CH_DELLINE	= $9C;	// delete line
	CH_INSLINE	= $9D;	// insert line

(* color defines *)

	COLOR_BLACK		= $00;
	COLOR_WHITE		= $0e;
	COLOR_RED		= $32;
	COLOR_CYAN		= $96;
	COLOR_VIOLET		= $68;
	COLOR_GREEN		= $c4;
	COLOR_BLUE		= $74;
	COLOR_YELLOW		= $ee;
	COLOR_ORANGE		= $4a;
	COLOR_BROWN		= $e4;
	COLOR_LIGHTRED		= $3c;
	COLOR_GRAY1		= $04;
	COLOR_GRAY2		= $06;
	COLOR_GRAY3		= $0a;
	COLOR_LIGHTGREEN	= $cc;
	COLOR_LIGHTBLUE         = $7c;

implementation

var
	RndSeed: smallint;


function ReadConfig(devnum: byte): cardinal; assembler;
{
DVSTAT
Byte 0 ($02ea):
Bit 0:Indicates the last command frame had an error. 
Bit 1:Checksum, indicates that there was a checksum error in the last command or data frame
Bit 2:Indicates that the last operation by the drive was in error. 
Bit 3:Indicates a write protected diskette. 1=Write protect 
Bit 4:Indicates the drive motor is on. 1=motor on 
Bit 5:A one indicates MFM format (double density) 
Bit 6:Not used 
Bit 7:Indicates Density and a Half if 1 

Byte 1 ($02eb):
Bit 0:FDC Busy should always be a 1 
Bit 1:FDC Data Request should always be 1 
Bit 2:FDC Lost data should always be 1 
Bit 3:FDC CRC error, a 0 indicates the last sector read had a CRC error 
Bit 4:FDC Record not found, a 0 indicates last sector not found 
Bit 5:FDC record type, a 0 indicates deleted data mark 
Bit 6:FDC write protect, indicates write protected disk 
Bit 7:FDC door is open, 0 indicates door is open 

Byte 2 ($2ec):
Timeout value for doing a format. 

Byte 3 ($2ed):
not used, should be zero 
}
asm
{	txa:pha

	lda devnum
	jsr @sio.devnrm
	bmi _err

	lda #'S'	; odczyt statusu stacji
	sta dcmnd

	jsr jdskint	; $e453
	bmi _err

	ldx <256	; 256 bajtow
	ldy >256	; w sektorze

	lda dvstat
	and #%00100000
	bne _skp

	ldx <128	;128 bajtow
	ldy >128	;w sektorze

_skp	jsr @sio.devsec 

	mva dvstat result
	mva dvstat+1 result+1
	mva dvstat+2 result+2
	mva dvstat+3 result+3

	ldy #0

_err	sty MAIN.SYSTEM.IOResult

	pla:tax
};
end;


procedure ReadSector(devnum: byte; sector: word; var buf); assembler;
asm
{	txa:pha

	lda devnum
	jsr @sio.devnrm
	bmi _err

	lda sector
	sta daux1
	lda sector+1
	sta daux2

	ldx buf
	ldy buf+1
	lda #'R'

	jsr @sio

_err	sty MAIN.SYSTEM.IOResult

	pla:tax
};  
end;


procedure WriteSector(devnum: byte; sector: word; var buf); assembler;
asm
{	txa:pha

	lda devnum
	jsr @sio.devnrm
	bmi _err

	lda sector
	sta daux1
	lda sector+1
	sta daux2

	ldx buf
	ldy buf+1
	lda #'P'

	jsr @sio

_err	sty MAIN.SYSTEM.IOResult

	pla:tax
};  
end;


procedure RunError(a: byte);
begin
	writeln('Runtime error ', a);
	halt;
end;


function Point(AX, AY: smallint): TPoint;
begin
  Result.X := AX;
  Result.Y := AY;
end;


function Rect(ALeft, ATop, ARight, ABottom: smallint): TRect;
begin
  Result.Left := ALeft;
  Result.Top := ATop;
  Result.Right := ARight;
  Result.Bottom := ABottom;
end;


function Bounds(ALeft, ATop, AWidth, AHeight: smallint): TRect;
begin
  Result.Left := ALeft;
  Result.Top := ATop;
  Result.Right := ALeft + AWidth;
  Result.Bottom :=  ATop + AHeight;
end;


function PointsEqual(const P1, P2: TPoint): Boolean; 
  begin
    { lazy, but should work }
    result:=cardinal(P1.x + P1.y shl 16)=cardinal(P2.x + P2.y shl 16);
  end;


function Peek(a: word): byte; register; assembler;
asm
{	ldy #0
	mva (edx),y Result
};
end;


function DPeek(a: word): word; register; assembler;
asm
{	ldy #0
	mva (edx),y Result
	iny
	mva (edx),y Result+1
};
end;


procedure Randomize; assembler;
asm
{	mva $d20a	RndSeed
	mva :rtclok+2	RndSeed+1
};
end;


function Random: Real; assembler; overload;
asm
{	txa:pha

_t	= edx

	LDA #$0

	sta result+1
	sta result+2
	sta result+3

	STA _T
RAN1	INC _T
	JSR POLY
	CMP #$0
	BEQ RAN1
	ORA #$10
	ldy #0
;

RAN2	INY
	JSR POLY
	ROL @
	ROL @
	ROL @
	ROL @
	AND #$F0
	STA _T+1
	JSR POLY
	ORA _T+1
	STA result-1,y
;	CPY #4
;	BCC RAN2
;	LDA _T
;	INY
;	STA result,y

	pla:tax

	jmp stop

POLY	TYA
	PHA
	LDY #$0
POLY1	INY
	CLC
	ROL POLYN
	ROL POLYN+1
	ROL POLYN+2
	ROL POLYN+3
	ROL POLYN+4
	ROL POLYN+5
	ROL POLYN+6
	ROL POLYN+7
	BCC POLY3
;
	LDX #$0
POLY2	LDA POLYN,X
	EOR GEN,X
	STA POLYN,X
	INX
	CPX #8
	BCC POLY2
	SEC
;
POLY3	ROL _T+2
	CPY #4
	BCC POLY1
;
	PLA
	TAY
	LDA _T+2
	AND #$0F
	CMP #$0A
	BCS POLY
	RTS
;
;
GEN	dta a($A1)
	dta $A2,$1A,$A2,$91,$C3,$93,$C0
;
POLYN	dta a($63)
	dta $42,$A1,$23,$55,$09,$03,$87

stop
};
end;


function Random(range: byte): byte; assembler; overload;
asm
{
;BYTE FUNC Rand(BYTE range)
; returns random number between 0 and
; range-1.  If range=0, then a random
; number between 0 and 255 is returned

	ldy	$d20a		; RANDOM

	lda	range
	beq	stop

	sta	ecx
	sty	eax

	lda	#0
	jsr	imulCL
	tay

stop	sty	Result
};
end;


function Random(range: smallint): smallint; overload;
begin

 if range = 0 then
	Result := 0
 else begin
  
	RndSeed := $4595 * RndSeed;
	Result := RndSeed mod range;

asm
{	lda range+1
	bpl plus

	lda Result+1
	bmi ok
	bpl sign
	
plus	lda Result+1
	bpl ok

sign	lda #0
	sub Result
	sta Result

	lda #0
	sbc Result+1
	sta Result+1
ok
}; 

  end;

end;


(*
function IntToReal(x: integer): Real; register; assembler;
//----------------------------------------------------------------------------------------------
// Convert an integer value to a real value
//----------------------------------------------------------------------------------------------
asm
{	mva edx+2 result+3
	mva edx+1 result+2
	mva edx result+1
	mva #0 result
};
end;
*)


function Abs(x: Real): Real; register; assembler; overload;
//----------------------------------------------------------------------------------------------
// Abs returns the absolute value of a variable.
// The result of the function has the same type as its argument, which can be any numerical type.
//----------------------------------------------------------------------------------------------
asm
{	lda edx+3
	spl
	jsr negEDX

	mva edx Result
	mva edx+1 Result+1
	mva edx+2 Result+2
	mva edx+3 Result+3
};
end;


function Abs(x: Integer): Integer; register; assembler; overload;
//----------------------------------------------------------------------------------------------
// Abs returns the absolute value of a variable.
// The result of the function has the same type as its argument, which can be any numerical type.
//----------------------------------------------------------------------------------------------
asm
{	lda edx+3
	spl
	jsr negEDX

	mva edx Result
	mva edx+1 Result+1
	mva edx+2 Result+2
	mva edx+3 Result+3
};
end;


function Sqr(x: Real): Real; overload;
//----------------------------------------------------------------------------------------------
// Sqr returns the square of its argument X
//----------------------------------------------------------------------------------------------
begin

 Result := x*x;

end;


function Sqr(x: integer): integer; overload;
//----------------------------------------------------------------------------------------------
// Sqr returns the square of its argument X
//----------------------------------------------------------------------------------------------
begin

 Result := x*x;

end;


function Sqrt(x: Real): Real; overload;
//----------------------------------------------------------------------------------------------
// Sqrt returns the square root of its argument X, which must be positive
//----------------------------------------------------------------------------------------------
var Divisor: Real;
begin
{ Hero's algorithm }

if x < 0.0 then exit;

Result  := x; 
Divisor := 1.0;

while Abs(Result - Divisor) > 0.01 do
  begin
  Divisor := (Result + Divisor) / 2.0;
  Result := x / Divisor;
  end;
end;


function Sqrt(x: Integer): Integer; overload;
//----------------------------------------------------------------------------------------------
// Sqrt returns the square root of its argument X, which must be positive.
//----------------------------------------------------------------------------------------------
var
  Divisor: Integer;
begin
{ Hero's algorithm }

if x < 0 then exit;

Result  := x; 
Divisor := 1;

while Abs(Result - Divisor) > 1 do
  begin
  Divisor := (Result + Divisor) shr 1;
  Result := x div Divisor;
  end;
end;


function ArcTan(value: real): real;
//----------------------------------------------------------------------------------------------
// Arctan returns the Arctangent of X, which can be any Real type. 
// The resulting angle is in radial units.
//----------------------------------------------------------------------------------------------
var x, y: real;
    sign: Boolean;
begin
  sign:=false;
  x:=value;
  y:=0.0;
 
  if (value=0.0) then begin
    Result:=0.0;
    exit;
  end else
   if (x < 0.0) then begin
    sign:=true;
    x:=-x;
   end;
   
  x:=(x-1.0)/(x+1.0);   
  y:=x*x;
  x := ((((((((.0028662257*y - .0161657367)*y + .0429096138)*y - 
             .0752896400)*y + .1065626393)*y - .1420889944)*y + 
             .1999355085)*y - .3333314528)*y + 1.0)*x;
  x:= .785398163397 + x;
  
  if sign then 
   Result := -x
  else
   Result := x;
 
end;


function Exp(x: Real): Real;
//----------------------------------------------------------------------------------------------
// Exp returns the exponent of X, i.e. the number e to the power X
//----------------------------------------------------------------------------------------------
var
  r: Real;
  k: byte;
begin
Result := 0.0;
r := 1.0;

for k := 1 to 8 do
  begin
  Result := Result + r;
  r := r * x / real(k);
  end;

end;


function Ln(x: Real): Real;
//----------------------------------------------------------------------------------------------
// Ln returns the natural logarithm of the Real parameter X. X must be positive
//----------------------------------------------------------------------------------------------
var
  r, a: Real;
  k: byte;
begin
Result := 0.0;
a := (x - 1.0) / (x + 1.0);
r := a;

for k := 1 to 8 do
  begin
  Result := Result + r / real(k shl 1 - 1);
  r := r * a * a;
  end;

Result := Result + Result;
end;


function FilePos(var f: file): cardinal; assembler;
asm
{	txa:pha

	mwa f bp2

	ldy #s@file.chanel
	lda (bp2),y
	tax
	lda #38
	sta iccmd,x
	jsr ciov

	sty IOResult

	mva icax3,x result
	mva icax4,x result+1
	mva icax5,x result+2
	mva #$00 result+3

	pla:tax
};
end;


procedure Seek(var f: file; a: cardinal); assembler;
asm
{	txa:pha

	mwa f bp2

	ldy #s@file.chanel
	lda (bp2),y
	tax
	lda #37
	sta iccmd,x

	mva a icax3,x
	mva a+1 icax4,x
	mva a+2 icax5,x

	jsr ciov

	sty IOResult

	pla:tax
};
end;


function Eof(var f: file): Boolean;
var i: cardinal;
    tmp: byte;
begin
	i:=FilePos(f);
asm
{	mwa f bp2

	ldy #s@file.record
	lda (bp2),y
	pha
	lda #1
	sta (bp2),y
	iny
	lda (bp2),y
	pha
	lda #0
	sta (bp2),y

};
	blockread(f, tmp, 1);

	Seek(f, i);
asm
{	ldy #s@file.record
	iny
	pla
	sta (bp2),y
	dey
	pla
	sta (bp2),y

	ldy #s@file.status
	lda (bp2),y
	and #e@file.eof
	sta Result
};
end;


function LowerCase(a: char): char;
//----------------------------------------------------------------------------------------------
// Converts a character to lowercase
//----------------------------------------------------------------------------------------------
begin

 case a of
  'A'..'Z': Result := chr(byte(a) + 32)
 else
  Result := a
 end;

end;


function UpCase(a: char): char;
//----------------------------------------------------------------------------------------------
// Converts a character to uppercase
//----------------------------------------------------------------------------------------------
begin

 case a of
  'a'..'z': Result := chr(byte(a) - 32)
 else
  Result := a
 end;

end;


procedure Val(const s: TString; var v: integer; var code: byte): assembler;
//----------------------------------------------------------------------------------------------
// Calculate numerical value of a string
//----------------------------------------------------------------------------------------------
asm
{	@StrToInt #adr.s

	tya
	pha

	mwa code bp2
	ldy #0

	pla
	sta (bp2),y

	mwa v bp2

	mva edx (bp2),y+
	mva edx+1 (bp2),y+
	mva edx+2 (bp2),y+
	mva edx+3 (bp2),y
};
end;


function FloatToStr(a: real): ^char; assembler;
//----------------------------------------------------------------------------------------------
// Convert a float value to a string
//----------------------------------------------------------------------------------------------
asm
{	txa:pha

	inx

	@ValueToStr #@printREAL

	mwa #@buf Result

	pla:tax
};
end;


procedure Str(a: integer; var s: TString); assembler;
//----------------------------------------------------------------------------------------------
// Convert a numerical value to a string
//----------------------------------------------------------------------------------------------
asm
{	txa:pha

	inx

	@ValueToStr #@printINT

	@move #@buf s #16	; !!! koniecznie przez wskaznik

	pla:tax
};
end;


procedure Poke(a: word; value: byte); register; assembler;
asm
{	ldy #0
	mva value (edx),y
};
end;


procedure DPoke(a: word; value: word); register; assembler;
asm
{	ldy #0
	mva value (edx),y
	iny
	mva value+1 (edx),y
};
end;


procedure FillChar(a: pointer; count: word; value: char); assembler; register; overload;
//----------------------------------------------------------------------------------------------
// Fills the memory starting at A with Count Characters with value equal to Value
//----------------------------------------------------------------------------------------------
asm
{	jsr @fill
};
end;

procedure FillChar(a: pointer; count: word; value: byte); assembler; register; overload;
//----------------------------------------------------------------------------------------------
// Fills the memory starting at A with Count Bytes with value equal to Value
//----------------------------------------------------------------------------------------------
asm
{	jsr @fill
};
end;

procedure FillChar(a: pointer; count: word; value: Boolean); assembler; register; overload;
//----------------------------------------------------------------------------------------------
// Fills the memory starting at A with Count Boolean with value equal to Value
//----------------------------------------------------------------------------------------------
asm
{	jsr @fill
};
end;

procedure FillChar(var x; count: word; value: char); assembler; register; overload;
asm
{	jsr @fill
};
end;

procedure FillChar(var x; count: word; value: byte); assembler; register; overload;
asm
{	jsr @fill
};
end;

procedure FillChar(var x; count: word; value: Boolean); assembler; register; overload;
asm
{	jsr @fill
};
end;


procedure FillByte(a: pointer; count: word; value: byte); assembler; register; overload;
//----------------------------------------------------------------------------------------------
// Fills the memory starting at A with Count Bytes with value equal to Value
//----------------------------------------------------------------------------------------------
asm
{	jsr @fill
};
end;

procedure FillByte(var x; count: word; value: byte); assembler; register; overload;
asm
{	jsr @fill
};
end;


procedure Move(source, dest: pointer; count: word); assembler; register; overload;
//----------------------------------------------------------------------------------------------
// Moves Count bytes from Source to Dest
//----------------------------------------------------------------------------------------------
asm
{	jsr @move
};
end;

procedure Move(var source, dest; count: word); assembler; register; overload;
asm
{	jsr @move
};
end;

procedure Move(var source; dest: pointer; count: word); assembler; register; overload;
asm
{	jsr @move
};
end;


function Sin(x: Real): Real; 
var
  r, x2: Real;
  k, a: byte;
begin

while x > M_PI_2 do x := x - M_PI_2;
while x < 0.0    do x := x + M_PI_2;

Result := 0.0;
r := x;
x2 := x * x;

for k := 1 to 9 do
  begin
  Result := Result + r;
  a := k shl 1;
  r := (-r) * x2 / real((a + 1) * a);
  end;

end;


function Cos(x: Real): Real;
begin

 Result := Sin(x + D_PI_2);

end;


function Space(b: Byte): ^char; assembler;
//----------------------------------------------------------------------------------------------
// Return a string of spaces
//----------------------------------------------------------------------------------------------
asm
{	ldy #0
	lda #' '
	sta:rne @buf,y+

	mva b @buf

	mwa #@buf Result
};
end;


function StringOfChar(c: Char; l: byte): ^char; assembler;
asm
{	ldy #0
	lda c
	sta:rne @buf,y+

	mva l @buf

	mwa #@buf Result
};
end;


procedure SetLength(var S: string; Len: byte); register; assembler;
asm
{	ldy #0
	mva Len (edx),y
};
end;


function BinStr(Value: cardinal; Digits: byte): TString; assembler;
asm
{	txa:pha

	ldy Digits
	cpy #32
	scc
	ldy #32

	sty @buf

_tob1	lda #0
	lsr Value+3
	ror Value+2
	ror Value+1
	ror Value
	adc #'0'
	sta @buf,y
	dey
	bne _tob1

	@move #@buf Result #33

	pla:tax
};
end;


function OctStr(Value: cardinal; Digits: byte): TString; assembler;
asm
{	txa:pha

	ldy Digits
	cpy #32
	scc
	ldy #32

	sty @buf

_toct1	ldx #3
	lda #0
_toct2	lsr Value+3
	ror Value+2
	ror Value+1
	ror Value
	ror @
	dex
	bne _toct2

	:5 lsr @

	ora #'0'
	sta @buf,y

	dey
	bne _toct1

	@move #@buf Result #33

	pla:tax
};
end;


function HexStr(Value: cardinal; Digits: byte): TString; register; assembler;
asm
{	txa:pha

	jsr @hexStr

	@move #@buf Result #33

	pla:tax
};
end;


procedure Pause; assembler; overload;
asm
{	lda:cmp:req :rtclok+2
};
end;


procedure Pause(n: word); assembler; overload;
asm
{	lda n
	ora n+1
	beq stop

	mwa n timcnt3

	lda #$FF
	sta timflg3
	lda:rne timflg3
stop
};
end;


function ParamCount: byte; assembler;
asm
{	@cmdline #255
	sta Result
};
end;


function ParamStr(i: byte): TString; assembler;
asm
{	@cmdline i
	@move #@buf Result #33
};
end;


function Concat(a,b: string): string; assembler; overload;
asm
{	mva #0 @buf
	@addString #adr.a
	@addString #adr.b
	@move #@buf #adr.Result #256
};
end;


function Concat(a: string; b: char): string; assembler; overload;
asm
{	mva #0 @buf
	@addString #adr.a
	inc @buf
	ldy @buf
	lda b
	sta @buf,y	
	@move #@buf #adr.Result #256
};
end;


function Concat(a: char; b: string): string; assembler; overload;
asm
{	mva #1 @buf
	lda a
	sta @buf+1
	@addString #adr.b	
	@move #@buf #adr.Result #256
};
end;


function Concat(a,b: char): string; overload;
begin
 Result[0]:=chr(2);
 Result[1]:=a;
 Result[2]:=b;
end;


end.
