{
  ZX Spectrum lib  
  Unit: zxlib.pas
    
  Author: Bostjan Gorisek (Gury)
  Initial release date: 26.3.2016
  Version: 1.0
  
  Supporting routines are copies of Mad Pascal library routines
}
unit zxlib;
  
interface

implementation

procedure Position(x,y: byte); assembler;
//----------------------------------------------------------------------------------------------
// Set cursor position on screen
//
// GotoXY positions the cursor at (X,Y), X in horizontal, Y in vertical direction relative to
// the origin of the current window. The origin is located at (1,1), the upper-left corner
// of the window.
//----------------------------------------------------------------------------------------------
asm
{	ldy x
	beq @+
	dey
	sty colcrs

@	ldy y
	beq @+
	dey
	sty rowcrs
@
};
end;

function Locate(x, y: smallint): byte; assembler;
//----------------------------------------------------------------------------------------------
// Return color of pixel
//----------------------------------------------------------------------------------------------
asm
{	txa:pha

	lda #$00

	cpw y main.system.ScreenHeight
	bcs stop

	mwa x colcrs
	mva y rowcrs

	lda #@IDget

	jsr @COMMAND

stop	sta Result

	pla:tax
};
end;

//-----------------------------------------------------------------------------
// Routine    : PrintAt
// Description: Print text on location y, x in text mode 0
//-----------------------------------------------------------------------------
procedure PrintAt(y, x : byte; text : string); overload;
begin
  Position(x+1, y+1);
  Write(text);
end;

//-----------------------------------------------------------------------------
// Routine    : PrintAt
// Description: Print text on location y, x to screen device S: 
//              (for example for text mode 1 and 2)
//-----------------------------------------------------------------------------
procedure PrintAt(var f : file; y, x : byte; text : string); overload;
begin  
  Position(x+1, y+1);  
  blockwrite(f, text[1], length(text));
end;

//-----------------------------------------------------------------------------
// Routine    : PrintAt
// Description: Print character on location y, x to screen device S:
//              (for example for text mode 1 and 2)
//-----------------------------------------------------------------------------
procedure PrintAt(var f : file; y, x : byte; text : char); overload;
begin  
  Position(x+1, y+1);
  blockwrite(f, text, 1);
end;

//-----------------------------------------------------------------------------
// Routine    : Flash
// Description: Print text on location y, x to screen device S: with
//              alternating text color change
//              (for example for text mode 1 and 2)
//-----------------------------------------------------------------------------
procedure Flash(var f : file; y, x, rep : byte; text : string);
var
  i, j : byte;
  text02 : string;
begin
    j := 0;
    text02 := text;

    for i := 1 to Length(text) do begin
      text02[i] := Chr(Ord(text02[i]) + 32);
    end;

    for i := 1 to rep do begin
      if j = 0 then begin
        j := 1;
        Position(x+1, y+1);
        blockwrite(f, text[1], length(text));
      end else begin
        j := 0;
        Position(x+1, y+1);
        blockwrite(f, text02[1], length(text));
      end;
      Pause(30);
    end;
end;

//-----------------------------------------------------------------------------
// Routine    : PrintAt
// Description: Print character on location y, x in text mode 0
//-----------------------------------------------------------------------------
procedure PrintAt(y, x : byte; c : char); overload;
begin
  Position(x+1, y+1);
  Write(c);
end;

//-----------------------------------------------------------------------------
// Routine    : Screen
// Description: Locate character on location y, x in text mode 0
//-----------------------------------------------------------------------------
function Screen(y, x : byte) : char; overload;
var
  loc : byte;
begin
  loc := Locate(x, y);
  result := chr(loc);
  PrintAt(y, x, result);
  //Position(x+1, y+1);
  //write(result);
end;

//-----------------------------------------------------------------------------
// Routine    : Screen
// Description: Locate character on location y, x from screen device S:
//              for use with text modes 1 and 2
//-----------------------------------------------------------------------------
function Screen(var f : file; y, x : byte) : char; overload;
var
  loc : byte;
begin
  loc := Locate(x, y);
  result := chr(loc);
  PrintAt(f, y, x, result);
end;

{
//-----------------------------------------------------------------------------
// Routine    : Beep
// Description: Sound in middle C
//-----------------------------------------------------------------------------
procedure Beep(delay : byte; c_note_diff : Integer);
var
  i : Integer;
begin
  Sound(0, 120+c_note_diff, 7, 10);
  Pause(40*delay);
end;
}

end.
